# -*- coding: utf-8 -*-
"""
Created on Sat Mar 13 16:06:52 2021

@author: Jens Skapski
"""

from cartopy.io.img_tiles import GoogleTiles
import cartopy.crs as ccrs
import matplotlib.pyplot as plt
import numpy as np
import math
import sys
from datetime import datetime
from shapely.geometry import Polygon, Point, LinearRing
from osgeo import gdal
import affine

##########################################################
##########################################################
###########Bitte nur folgende Werte verändern!############
##########################################################
##########################################################

ML = 6.5 # Magnitude, Stärke des Erdbebens
D = 10  # Tiefe des Erdbebens in Kilometern
lat1,lon1 =47.5,10 # Koordinaten des Epizentrums (Breite, Länge)
angle2 = 90 # Winkel des Bruchs (in °), nur bei hohen Magnituden relevant  

##########################################################
##########################################################
##########Ab hier am Code nichts mehr verändern!##########
##########################################################
##########################################################

latmin = lat1-1.2
latmax = lat1+1.2
lonmin = lon1-2.3
lonmax = lon1+2.3
print("Die Berechnung läuft. Dies kann bis zu ca. 30 Sekunden dauern. In der Zeit das Programm bitte nicht beenden!")
fig, ax = plt.subplots(figsize=(17,17))  
ds = gdal.Open('Deutschland.tif')
elevation = np.array(ds.GetRasterBand(1).ReadAsArray())
indices = np.where(elevation >= -12200)
affine_transform = affine.Affine.from_gdal(*ds.GetGeoTransform())
lon, lat = affine_transform * np.flip(indices)
altit, altitude,elevlon,elevlat = [],[],[],[]
index = np.where((lat>=latmin-0.1) & (lat<=latmax+0.1) & (lon>=lonmin-0.1) & (lon<=lonmax+0.1))[0]
elevation = np.flip(elevation)
MD = math.log10(D)
I01 = (ML + 0.154 - (0.555 * MD)) / 0.636 #Rudloff & Leydecker 2002
I02 = (ML + 0.87 - (0.78 * MD)) / 0.74 #Grünthal & Wahlström 2003
if D < 1:
    print("Fehler: Bitte eine gültige Tiefe angeben (mindestens 1 km)")
    sys.exit()
elif D < 5:
    I0 = I02
    I0 = I0 + (5-D)/3
    abskoef = 0.003 + (5-D)/120
elif D > 30:
    print("Fehler: Bitte eine realistische Tiefe angeben (maximal 30 km)")
    sys.exit()
else:
    I0 = I01
    abskoef = 0.003
class ShadedReliefESRI(GoogleTiles):
    def _image_url(self, tile):
        x, y, z = tile
        url = ('https://server.arcgisonline.com/ArcGIS/rest/services/' \
               'World_Topo_Map/MapServer/tile/{z}/{y}/{x}.jpg').format(
               z=z, y=y, x=x)
        return url
ax = plt.axes(projection=ShadedReliefESRI().crs)
extent = [lonmin, lonmax, latmin, latmax]
ax.set_extent(extent, crs=ccrs.Geodetic())
ax.add_image(ShadedReliefESRI(), 8)
gl = ax.gridlines(draw_labels=True, linewidth=0.1)
gl.xlabels_top=None
gl.ylabels_left=None
distrand,Ish,Ish2,Shake,Shake2,Shakelon,Shakelat,point = [],[],[],[],[],[],[],[] 
rupt = math.pow(10,(ML-4.38)/1.49)/2 
lat1x = math.radians(lat1)
lon1x = math.radians(lon1) 
brng = angle2 * math.pi/180
brng2 = (angle2+180)*math.pi/180
lat2 = math.asin( math.sin(lat1x)*math.cos(rupt/6378.1) +
     math.cos(lat1x)*math.sin(rupt/6378.1)*math.cos(brng))
lon2 = lon1x + math.atan2(math.sin(brng)*math.sin(rupt/6378.1)*math.cos(lat1x),
             math.cos(rupt/6378.1)-math.sin(lat1x)*math.sin(lat2))
lat2 = math.degrees(lat2)
lon2 = math.degrees(lon2)
lat2x = math.asin( math.sin(lat1x)*math.cos(rupt/6378.1) +
     math.cos(lat1x)*math.sin(rupt/6378.1)*math.cos(brng2))
lon2x = lon1x + math.atan2(math.sin(brng2)*math.sin(rupt/6378.1)*math.cos(lat1x),
             math.cos(rupt/6378.1)-math.sin(lat1x)*math.sin(lat2x))
lat2x = math.degrees(lat2x)
lon2x = math.degrees(lon2x)
polygon = Polygon([(lat2,lon2),(lat2,lon2),(lat2x,lon2x),(lat2x,lon2x)])  
for j in range(0,len(elevation)):
    for i in range(0,len(elevation[j])):
        altit.append(elevation[j][i])
for i in index:
    yipp = altit[i]
    altitude.append(yipp)
for i in index:
    loo = lon[i]
    elevlon.append(loo)
for i in index: 
    laa = lat[i]
    elevlat.append(laa)
elevlon = np.array(elevlon)
elevlat = np.array(elevlat)
altav = max(altitude)-min(altitude)
if altav >= 800:
    altav = 800
else:
    altav = altav
for i in range(len(altitude)):
    latmax2 = (elevlat[i] + 0.025)
    latmin2 = (elevlat[i] - 0.025)
    lonmax2 = (elevlon[i] + 0.025)
    lonmin2 = (elevlon[i] - 0.025)
    index2 = np.where((elevlat>latmin2) & (elevlat<latmax2) & (elevlon>lonmin2) & (elevlon<lonmax2))[0]
    alt = []
    for j in index2:
        alt.append(altitude[j])
    if max(alt) == 0:
        slope = 1
    else:
        if min(alt) == 0:
            slope = 1
        else:
            slope = 1-((max(alt)-min(alt))/altav)
    point = Point(elevlat[i], elevlon[i])
    pol_ext = LinearRing(polygon.exterior.coords)
    d = pol_ext.project(point)
    p = pol_ext.interpolate(d)
    closest_point_coords = list(p.coords)
    elevla = [x[0] for x in closest_point_coords][0]
    elevlo = [x[1] for x in closest_point_coords][0]
    distx2 = abs((elevlon[i]-elevlo) * math.pi / 180)
    disty2 = abs((elevlat[i]-elevla) * math.pi / 180)
    lat111 = elevla * math.pi / 180
    lat222 = (elevlat[i]) * math.pi / 180
    a2 = math.sin(disty2/2)**2 + math.cos(lat111) * math.cos(lat222) * math.sin(distx2/2)**2
    distrand = 2 * 6371 * math.asin(math.sqrt(a2))
    Za = 3 + ((elevlat[i]-47)/4)
    alti = ((3*np.mean(alt))+222)/4
    altdiff = altitude[i]-alti
    if altdiff >= 1000:
        altI = 1000 / 222
    elif altdiff <= -1000:
        altI = -1000 / 222
    else:
        altI = altdiff / 222
    ela = elevlat[i] - lat1
    elo = elevlon[i] - lon1
    if slope <= 0.65:
        sl = 0.65
    elif slope >= 0.9:
        sl = slope + 2*(slope-0.95) + 0.02
    else:
        sl = slope + 0.02
    if I0 >= 10:
        I0 = 10
    else:
        I0 = I0
    I0app =-0.5 - (D-5)/22 
    Ish2 = (I0+I0app) - 3*math.log10((1/D)*math.sqrt((D**2+(distrand+1)**2)))-1.3*abskoef*(math.sqrt(1+(D**2+(distrand+1)**2)) - D) + math.sin(math.radians(40-(40/(distrand+1))))*(0.00001)
    Ish2 = Ish2*(sl+((distrand/(500+(D*2)))*sl))
    IshB = (I0+I0app) - 3*math.log10((1/D)*math.sqrt((D**2+(distrand+1)**2)))-1.3*abskoef*(math.sqrt(1+(D**2+(distrand+1)**2)) - D) - 0.5*(1+(math.sqrt((distrand*distrand)+(D*D))/(ML*50)))*(altI*ML/10)+math.sin(math.radians(40-(40/(distrand+1))))*0.00001
    if altitude[i] < -15:
        Ish = 0
        IshB = 0
        IX = 0
    else:
        Ish = Ish2
        IshB = IshB
        IX = (Ish+IshB)/2

    Shake2.append(IX)    
    if IX >= 9:
        IX = 8.99
    else:
        IX = IX
    if IX <= 1:
        IX = 1
    else:
        IX = IX
    Shake.append(IX)
    Shakelon.append(elevlon[i])
    Shakelat.append(elevlat[i])
cont = [2,3,4,5,6,7,8,9]
xshake,yshake = Shakelon,Shakelat
Isoseists = [2,2.5,3,3.5,4,4.5,5,5.5,6,6.5,7,7.5,8,8.5,9]
cnt = plt.tricontourf(np.array(xshake),np.array(yshake),np.array(Shake),Isoseists,cmap=plt.cm.get_cmap("jet",lut=14), alpha=0.4,transform=ccrs.Geodetic(), zorder = 100)
cnt.set_clim(2,9)
cnt.cmap.set_under('white')
cnt.cmap.set_over('white')
cb = plt.colorbar(spacing='uniform', ticks=[2,3,4,5,6,7,8,9],orientation="horizontal", fraction=0.06, pad=0.04)
cb.set_label("Intensitätsskala (EMS-98)", fontsize=20, color='black')
cb.ax.set_xticklabels(["kaum spürbar (2)\nkeine Schäden", 'schwach (3)\nkeine Schäden', 'deutlich (4)\nkeine Schäden','mäßig (5)\nkaum Schäden','stark (6)\ngeringe Schäden','sehr stark (7)\nmäßige Schäden','sehr stark (8)\nschwere Schäden','extrem (9+)\nzerstörend'], fontsize=14, color='black')
now = datetime.now().strftime('%d.%m.%Y %H:%M:%S')
print("Höchste Intensität: %s" % round(max(Shake),1))
plt.text(lonmin+(1.2/100),latmin+(1.2/100),"Quellcode: Jens Skapski, erdbebennews.de. Berechnung: Privater Nutzer. Dargestellt ist die berechnete Maximalintensität. Durch lokale Effekte ergibt sich eine\nUnsicherheit von +/- 1. Intensitäten unter 4 (IV) werden in der Regel nicht von allen Anwohnern verspürt. Intensität 2 (II) wird nur von sehr wenigen verspürt. \nErstellt: %s Mitteleuropäische Zeit\nMap tiles: Esri, HERE, Garmin, Intermap, increment P Corp., GEBCO, USGS, FAO, NPS, NRCAN, GeoBase, IGN, Kadaster NL, Ordnance Survey,\nEsri Japan, METI, Esri China (Hong Kong), (c) OpenStreetMap contributors, and the GIS User Community" % (now), zorder=10, size=10, color="black",bbox=dict(boxstyle='round', alpha=0.5, facecolor="whitesmoke", edgecolor="whitesmoke", Clip_on=True),transform=ccrs.Geodetic())
